"""
$RCSfile: testCVSSandboxRegistry.py,v $
Unit tests for CVSSandboxRegistry object within the CVSFile product

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

import os, sys, tempfile
execfile(os.path.join(sys.path[0], 'framework.py'))

import Zope
from Products.CVSFile.CVSSandboxRegistry import CVSSandboxRegistry, SandboxInfo

class TestCVSSandboxRegistry(unittest.TestCase):

    def setUp(self):
        self._reg = CVSSandboxRegistry("qa", "QA Sandboxes",
                                       "These are the sandboxes for testing",
                                       SandboxInfo("CKS test1",
                                                   "/home/cstrong/ade/",
                                                   "/repository/cvsroot"))

    def testSimpleCreate(self):
        """
        
        Create registry, verify there is one and only one sandbox,
        and verify that it is the default sandbox
        
        """
        assert self._reg.numSandboxes() == 1

        si = self._reg.defaultSandbox
        assert si.name    == "CKS test1"
        assert si.basedir == "/home/cstrong/ade/"
        assert si.cvsroot == "/repository/cvsroot"

        si2 = self._reg.getSandboxByName("CKS test1")
        assert si == si2


    def testDuplicateSandbox(self):
        "Try to register a sandbox with the same name as one already registered"
        try:
            self._reg.addSandbox(SandboxInfo("CKS test1", "is dupe", "will fail"))
        except:
            pass
        else:
            fail('Expected a ValueError indicating duplicate sandbox')

    def testAddSandbox(self):
        "Try to add a second sandbox.  "
        si = self._reg.getSandboxByName("CKS test1")
        assert si is not None
        si2 = self._reg.getDefaultSandbox()
        assert si == si2

        si = SandboxInfo("WGM dev", "/home/cmorris/ade",
                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot")
        self._reg.addSandbox(si)

        si3 = self._reg.getDefaultSandbox()
        assert si != si3
        assert self._reg.numSandboxes() == 2
        list = self._reg.sandboxList()
        assert len(list) == 2

        self._reg.setDefaultSandbox("WGM dev")

        si4 = self._reg.getSandboxByName("WGM dev")
        si5 = self._reg.getDefaultSandbox()
        assert si4 == si5
        assert self._reg.isDefault("WGM dev")

    def testSetDefaultToNonExistent(self):
        "Try to set the default sandbox to one that does not exist"
        try:
            self._reg.setDefaultSandbox("i don't exist")
        except:
            pass
        else:
            fail('Expecting a ValueError indicating a nonexistent sandbox')

    def testRemoveSandbox(self):
        "Ensure that a sandbox can be removed"
        assert self._reg.numSandboxes() == 1
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        assert self._reg.numSandboxes() == 2
        self._reg.addSandbox(SandboxInfo("WGM dev2", "/home/cmorris/ade",
                             ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        assert self._reg.numSandboxes() == 3

        self._reg.removeSandbox("WGM dev")
        assert self._reg.numSandboxes() == 2
        
    def testRemoveNonExistentSandbox(self):
        "Attempt to remove nonexistent sandbox"
        assert self._reg.numSandboxes() == 1
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                             ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        assert self._reg.numSandboxes() == 2
        self._reg.addSandbox(SandboxInfo("WGM dev2", "/home/cmorris/ade",
                             ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        assert self._reg.numSandboxes() == 3

        try:
            self._reg.removeSandbox("i dont exist")
        except:
            pass
        else:
            fail('Expecting a ValueError indicating a nonexistent sandbox')

        assert self._reg.numSandboxes() == 3

    def testRemoveDefaultSandbox(self):
        "Attempt to remove the default sandbox"
        assert self._reg.numSandboxes() == 1
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                             ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        assert self._reg.numSandboxes() == 2
        self._reg.addSandbox(SandboxInfo("WGM dev2", "/home/cmorris/ade",
                             ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        assert self._reg.numSandboxes() == 3

        self._reg.setDefaultSandbox("WGM dev")
        
        try:
            self._reg.removeSandbox("WGM dev")
        except:
            pass
        else:
            fail('Expecting a ValueError indicating default cannot be removed')

        assert self._reg.numSandboxes() == 3

    def testChangeNonExistentSandbox(self):
        "Attempt to change a nonexistent sandbox"
        try:
            self._reg.changeSandbox("WGM dev",
                                    "foo", "bar", "mumble")
        except:
            pass
        else:
            fail('Expecting a ValueError indicating a nonexistent sandbox')

    def testChangeSandbox(self):
        "Attempt to change a sandbox"
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                             ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))

        self._reg.changeSandbox("WGM dev",
                                SandboxInfo("foo", "bar", "mumble"))

        assert not self._reg.getSandboxByName("WGM dev")
        si = self._reg.getSandboxByName("foo")
        assert si.name    == "foo"
        assert si.basedir == "bar"
        assert si.cvsroot == "mumble"

    def testChangeToDupSandbox(self):
        "Attempt to change a sandbox to a name that already exists"
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                             ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))

        try:
            self._reg.changeSandbox("WGM dev",
                                    SandboxInfo("CKS test1", "bar", "mumble"))
        except:
            pass
        else:
            fail('Expected a ValueError indicating duplicate sandbox')
        
# How to test selection?
# create registry, ask which one is selected (none)
# add another one
# select the first, verify that it is selected
# make the second one the default
# delete the first one
# ask which one is selected (none)
# select the (now) first one, verify its selected
# create eight more, now select the last one, verify its selected

# ACTIONS:
# set a sandbox to be the default
# delete one or more sandboxes
# set a sandbox to be selected
# change a sandbox

framework()
